# This file was automatically generated by SWIG (http://www.swig.org).
# Version 2.0.4
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

package Math::GSL::Version;
use base qw(Exporter);
use base qw(DynaLoader);
package Math::GSL::Versionc;
bootstrap Math::GSL::Version;
package Math::GSL::Version;
@EXPORT = qw();

# ---------- BASE METHODS -------------

package Math::GSL::Version;

sub TIEHASH {
    my ($classname,$obj) = @_;
    return bless $obj, $classname;
}

sub CLEAR { }

sub FIRSTKEY { }

sub NEXTKEY { }

sub FETCH {
    my ($self,$field) = @_;
    my $member_func = "swig_${field}_get";
    $self->$member_func();
}

sub STORE {
    my ($self,$field,$newval) = @_;
    my $member_func = "swig_${field}_set";
    $self->$member_func($newval);
}

sub this {
    my $ptr = shift;
    return tied(%$ptr);
}


# ------- FUNCTION WRAPPERS --------

package Math::GSL::Version;


# ------- VARIABLE STUBS --------

package Math::GSL::Version;

*GSL_VERSION = *Math::GSL::Versionc::GSL_VERSION;
*GSL_MAJOR_VERSION = *Math::GSL::Versionc::GSL_MAJOR_VERSION;
*GSL_MINOR_VERSION = *Math::GSL::Versionc::GSL_MINOR_VERSION;
1;
